function [SOLN]=NN_GA_opt;
%
%Function to perform the surrogate model random search algorithm for
%optimization of compuatationally expensive functions
%Created by:  John C. Brigham
%Last Update:  4/26/2007 JCB
%
tic
%Load the experimental data
test_data=load('test_data.txt');    %already normalized
%test_data=test_data_dum';
%
NR=length(test_data);  %Automatically chosen as # of analysis outputs
%
%Define the optimization parameters
%************************************************************************
%
%Specify SMARS parameters
NI=4;   %Number of unknown parameters in problem (here E and NU, g=k, TAU)

tolerance=0.1;   %Solution tolerance to determine whether to stop search
num_it=25;       %Number of iterations for search method
global minp;    %vector to hold minimum parameter values
global maxp;    %vector to hold maximum parameter values
minp=[0.001,0,0,0];
maxp=[0.3,1,1,1];
new_itrain=0;     %set to 1 if creating a new initial training set, 0 otherwise
num_itrain=40;  %# of initial sets generated uniformly over each range
itrain_distn=[minp;maxp];   %matrix of ranges for itrain distribution
%
%Specify surrogate model parameters (Neural network parameters)
%SMSMSMSMSMSMSMSMSMSMSMSMSMSMSMSMSMSMSMSMSMSMSMSMSMSMSMSMSMSMSMSMSMSMSMSMSM
use_SM = 1;     %specify 1 if using surrogate model method 0 if not
global HN;
HN=8;  %Number of NN hidden nodes for 2 hidden layers
global min_train;
global max_HN;
min_train=1e-4; %minimum training error to reach otherwise hidden nodes are addded
max_HN=60;  %maximum number of hidden nodes used in the NN
global epochs;
global showepoch;
global trainerror;
epochs=15000;   %Maximum number of epochs for NN training
showepoch=NaN; %values of epoch number to show NN training error
trainerror=1e-6;    %minimum error to train NN to.
err_max=200000;   %maximum value of error for NN training set
p_window=0; %set to 1 if using % of best, 0 if using % of search space
nn_maxp=0.25;    %percent of range or best to use for SM training set window centered about the best
%initialize global variables
global net;     %neural network object
global mxi;     %normalizers for neural network input
global mxj;     %normalizers for neural network output
global nn_minv; %vector based on minimum training set inputs
global nn_maxv; %vector based on maximum training set outputs
%SMSMSMSMSMSMSMSMSMSMSMSMSMSMSMSMSMSMSMSMSMSMSMSMSMSMSMSMSMSMSMSMSMSMSMSMSM
%
%Specify global search parameters (Random Search parameters)
%RSRSRSRSRSRSRSRSRSRSRSRSRSRSRSRSRSRSRSRSRSRSRSRSRSRSRSRSRSRSRSRSRSRSRSRSRS
num_rand=5;    %number of random numbers to generate with RS
p_dev=0;    %set to 1 if using % of best, 0 if using % of search space 
dev_mult=0.25;   %multiplier of mean value for RS gaussian std. dev.
num_s_pole = 1; %the number of search poles in addition to best set
percent_pole = 0.2; %percentage of top sets to choose search pole
set_proxy = 0.01;  %average percentage parameters can be close for RS
global num_RS_loop;
num_RS_loop=20; %maximum number of trials to obtain new RS set before uniform distb. is taken
%RSRSRSRSRSRSRSRSRSRSRSRSRSRSRSRSRSRSRSRSRSRSRSRSRSRSRSRSRSRSRSRSRSRSRSRSRS
%
%************************************************************************
%
%Initialize parameters for algorithm monitoring
k=1;
FINISH=0;
%
%Generate and load the initial randomly generated data
if (new_itrain == 1)
    get_train_data(num_itrain,itrain_distn);
end
train_data1=load('itrain.txt');  %not normalized in order of response then parameters
%
%evaluate the error of the initial parameter sets and create matrix
%of params and error
train_data2(:,1:NI)=train_data1(:,1:NI);
for i=1:length(train_data1(:,1))
    [train_data2(i,NI+1)]=get_Error(test_data,train_data1(i,NI+1:NI+NR),train_data1(i,1:NI));
end
%
%Determine the best initial solution and corresponding error
err_mat=sortrows(train_data2,NI+1);
RS_BEST(1,:)=err_mat(1,1:NI);
RS_ERROR(1,1)=err_mat(1,NI+1);
%
%if using surrogate-model method then perform SM opt
if (use_SM == 1)
    %
    %update the min and max used for GA with the NN
    if (p_window == 1)
        nn_minv=RS_BEST-nn_maxp*abs(RS_BEST);
        nn_maxv=RS_BEST+nn_maxp*abs(RS_BEST);
    else
        nn_minv=RS_BEST-nn_maxp*abs(maxp-minp);
        nn_maxv=RS_BEST+nn_maxp*abs(maxp-minp);
    end
    %
    %Determine Distance of parameters from the current best and check if within
    %surrogate model window
    for i=1:length(train_data2(:,1))
        check=0;
        for j=1:NI
            if ((train_data2(i,j) > nn_maxv(j)) | (train_data2(i,j) < nn_minv(j)))
                check=1;
            end
        end
        train_data3(i,1:NI)=train_data2(i,1:NI);
        train_data3(i,NI+1)=check;
    end
    %
    %if the error is below the max error and distance is less than and greater 
    %than window, then add to the training set for NN
    j=1;
    for i=1:length(train_data2(:,1))
        if (train_data2(i,NI+1) < err_max) & (train_data3(i,NI+1) == 0)
            train_data(j,:)=train_data1(i,:);
            j=j+1;
        end
    end
    dlmwrite('train_data.txt',length(train_data(:,1)),'precision','%.8e');
    %
    %perform surrogate model optimization
    [out_data,ga_err_guess(1,1)]=SM_opt(train_data,NI,NR);
    ga_guess(1,:)=out_data;
    dlmwrite('SM_guess.txt',ga_guess(1,:),'precision','%.8e');
    %
    %perform a sim with the new parameter set and get actual responses
    [new_train1]=sim_fnct(out_data);
    %
    %get the response error for the SM estimate and check if below tol
    train_data1(length(train_data1(:,1))+1,:)=new_train1;
    new_train2(1,1:NI)=new_train1(1,1:NI);
    [new_train2(1,NI+1)]=get_Error(test_data,new_train1(1,NI+1:NI+NR),new_train1(1,1:NI));
    train_data2(length(train_data2(:,1))+1,:)=new_train2;
    ga_error(1,1)=new_train2(1,NI+1);
    dlmwrite('SM_error.txt',ga_error(1,1),'precision','%.8e');
end
%
%Determine the current best solution and corresponding error
err_mat=sortrows(train_data2,NI+1);
BEST(1,:)=err_mat(1,1:NI);
ERROR(1,1)=err_mat(1,NI+1);
dlmwrite('current_soln.txt',BEST(1,:),'precision','%.8e');
dlmwrite('current_error.txt',ERROR(1,1),'precision','%.8e');
%
%determine if there is a solution already present
if (ERROR(1,1) < tolerance)
    FINISH = 1;
    SOLN = BEST(1,:);
end
%
%if the solution is not there perform SMARS/RS optimization
while (FINISH == 0) 
    %
    %find the other sets than best to use for random search as search poles
    [best_set]=search_pole(err_mat,NI,num_s_pole,percent_pole);
    %
    %generate new sets of parameters around the search poles
    if (p_dev == 1)
        dev_v=abs(dev_mult*BEST(k,:));
    else
        dev_v=abs(dev_mult*(maxp-minp));
    end
    [RS_train_data1(1:num_rand,:)]=RS_data(BEST(k,:),num_rand,NI,dev_v,train_data2,set_proxy);
    for ii=1:num_s_pole
        if (p_dev == 1)
            dev_v=abs(dev_mult*best_set(ii,:));
        else
            dev_v=abs(dev_mult*(maxp-minp));
        end
        [RS_train_data1(ii*num_rand+1:(ii+1)*num_rand,:)]=RS_data(best_set(ii,:),num_rand,NI,dev_v,train_data2,set_proxy);
    end
    %
    %determine the error of the new training sets
    RS_train_data2(:,1:NI)=RS_train_data1(:,1:NI);
    for i=1:length(RS_train_data1(:,1))
        [RS_train_data2(i,NI+1)]=get_Error(test_data,RS_train_data1(i,NI+1:NI+NR),RS_train_data1(i,1:NI));
    end
    %add training data sets to various matrices
    for i=1:length(RS_train_data1(:,1))
        train_data1(length(train_data1(:,1))+1,:)=RS_train_data1(i,:);
    end
    for i=1:length(RS_train_data2(:,1))
        train_data2(length(train_data2(:,1))+1,:)=RS_train_data2(i,:);
    end
    %
    %Determine the current best solution and corresponding error
    err_mat=sortrows(train_data2,NI+1);
    RS_BEST(k+1,:)=err_mat(1,1:NI);
    RS_ERROR(k+1,1)=err_mat(1,NI+1);
    %
    %if using surrogate model optimization perform iteration of SM opt
    if (use_SM == 1)
        %
        %update the min and max used for the SM optimization
        if (p_window == 1)
            nn_minv=RS_BEST(k+1,:)-nn_maxp*abs(RS_BEST(k+1,:));
            nn_maxv=RS_BEST(k+1,:)+nn_maxp*abs(RS_BEST(k+1,:));
        else
            nn_minv=RS_BEST(k+1,:)-nn_maxp*abs(maxp-minp);
            nn_maxv=RS_BEST(k+1,:)+nn_maxp*abs(maxp-minp);
        end
        %
        %Determine Distance of the parameters from the current best
        for i=1:length(train_data2(:,1))
            check=0;
            for j=1:NI
                if ((train_data2(i,j) > nn_maxv(j)) | (train_data2(i,j) < nn_minv(j)))
                    check=1;
                end
            end
            train_data3(i,1:NI)=train_data2(i,1:NI);
            train_data3(i,NI+1)=check;
        end
        %
        %if the error is below the max error and distance is less than and 
        %greater than window, then add to the training set for SM
        train_data=[];
        j=1;
        for i=1:length(train_data2(:,1))
            if (train_data2(i,NI+1) < err_max) & (train_data3(i,NI+1) == 0)
                train_data(j,:)=train_data1(i,:);
                j=j+1;
            end
        end
        dlmwrite('train_data.txt',length(train_data(:,1)),'-append','precision','%.8e');
        %
        %perform surrogate model optimization
        [out_data,ga_err_guess(k+1,1)]=SM_opt(train_data,NI,NR);
        ga_guess(k+1,:)=out_data;
        dlmwrite('SM_guess.txt',ga_guess(k+1,:),'-append','precision','%.8e');
        %
        %perform a sim with the new parameter set and get actual responses
        [new_train1]=sim_fnct(out_data);
        %
        %get the response error for the SM estimate and check if below tol
        train_data1(length(train_data1(:,1))+1,:)=new_train1;
        new_train2(1,1:NI)=new_train1(1,1:NI);
        [new_train2(1,NI+1)]=get_Error(test_data,new_train1(1,NI+1:NI+NR),new_train1(1,1:NI));
        train_data2(length(train_data2(:,1))+1,:)=new_train2;
        ga_error(k+1,1)=new_train2(1,NI+1);
        dlmwrite('SM_error.txt',ga_error(k+1,1),'-append','precision','%.8e');
    end
    %
    %find the "best" set of parameters found so far and corresponding error
    err_mat=sortrows(train_data2,NI+1);
    BEST(k+1,:)=err_mat(1,1:NI);
    ERROR(k+1,1)=err_mat(1,NI+1);
    dlmwrite('current_soln.txt',BEST(k+1,:),'-append','precision','%.8e');
    dlmwrite('current_error.txt',ERROR(k+1,1),'-append','precision','%.8e');
    %
    %determine if there is a solution present or iterations maxed
    if (ERROR(k+1,1) < tolerance) | (k >= num_it)
        FINISH = 1;
        SOLN = BEST(k+1,:);
    end
    %
    %increment k
    k=k+1;
end
%
%save the output data
save SOLN SOLN;
save train_data1 train_data1;
save train_data2 train_data2;
save ERROR ERROR;
save BEST BEST;
save RS_BEST RS_BEST;
save RS_ERROR RS_ERROR;
if (use_SM == 1)
    save train_data train_data;
    save train_data3 train_data3;
    save ga_guess ga_guess;
    save ga_error ga_error;
    save ga_err_guess ga_err_guess;
    save NNa net;
    save mxi mxi;
    save mxj mxj;
    save HN HN;
end
toc