"""
odbdisplacement.py

This script will get displacements from ODB
Programmer: John Brigham
Created: 4/18/05
Last Update: 4/18/05 John Brigham
See Section 8 of ABAQUS Scripting Manual
Version 6.5
May need to use: 
party=odb.rootAssembly.instances['InstanceName']
odb.steps['Step-1'].frames[1].fieldOutputs['POR'].getSubset(region=party).values[nodesLabel].data
if multiple parts with similar data

"""
# These two lines must be in every python code in ABAQUS    
from odbAccess import *

from abaqusConstants import *

# Open the file that contains the node numbers to be read from the output file
myInputFile  = open('Nodes','r')

# Open the file that the output of nodes, displacement and time will be written to
outputFile = open('pressure.txt','w+')

# Read the nodes from the input file
Nodes = myInputFile.readlines()

# Open the odb (output data base) file for the current project ViscoSolid
odb = openOdb(path='plane_chalk_fluid_mesh2.odb')

# Determine the number of time steps available in Step-1 of the analysis
lengther=len(odb.steps['Step-1'].frames)

#sX='%.4E' % odb.steps['Step-1'].frames[1].fieldOutputs['POR'].values[3259].data
#sX='%s' %Nodes
#outputFile.write(sX)

# Loop through the measurement nodes and the time steps
# Obtain the data required (node,time,xyz disp) and write to the file
for j in range(len(Nodes)):
  numNode = int(Nodes[j])-1
  for i in range(lengther-1):
    sTime = '%.6E' % odb.steps['Step-1'].frames[i+1].frameValue
    real = '%.4E' % odb.steps['Step-1'].frames[i+1].fieldOutputs['POR'].values[numNode].data
    complex = '%.4E' % odb.steps['Step-1'].frames[i+1].fieldOutputs['POR'].values[numNode].conjugateData
    sNode = '%s' % odb.steps['Step-1'].frames[i+1].fieldOutputs['POR'].values[numNode].nodeLabel
    outputFile.write(sNode+'\t'+sTime+'\t'+real+'\t'+complex+'\n')

# Close output file
outputFile.close()
myInputFile.close()